## The GENERATE Tool is created by NA-ACCORD for local address geocoding and 
## submission of area-Level indicators

# This script is as an alternative to "fips_generator.R"
# Instead of using FCC FIPS API converter (https://geo.fcc.gov/api/census/), it uses tigris::call_geolocator_latlon to generate FIPS codes from latitude and longitude 
# However note, the FIPS codes generated at the census tract level and the block level may not be the same for "fips_generator.R" and "fips_gen_alternative.R"
# “fips_gen_alternative.R” is based on https://geocoding.geo.census.gov/geocoder/geographies/coordinates 

# User Instructions:
# If you obtained your geocoded addresses not by running "geocoding.R", please make sure your geocoded addresses have the following format:
# -The geocoded addresses are in a csv file named "adrs_geocoded.csv"
# -The "adrs_geocoded.csv" file must contain: 
# id (unique identifier of the input addresses)
# lat (latitude)
# lon (longitude)
# month (month of the input address)
# day (day of the input address)
# year (year of the input address)
# ZIP (zip code of the input addresses)
# -If you have the following information, please also include them in the "adrs_geocoded.csv" file:
# state_matched (value is "TRUE" or "FALSE" to indicate whether the state part of the addresses are matched after geocoding)
# city_matched (value is "TRUE" or "FALSE" to indicate whether the city part of the addresses are matched after geocoding)
# zip_matched (value is "TRUE" or "FALSE" to indicate whether the zip code part of the addresses are matched after geocoding)
# street_matched (value is "TRUE" or "FALSE" to indicate whether the street part of the addresses are matched after geocoding)
# The name of the columns are: id, lat, lon, state_matched, city_matched, zip_matched, street_matched, month, day, year, ZIP
# If you geocoded addresses by running "geocoding.R", please ignore these instructions and start from the code below

### In this step, we use perturbed longitude and latitude for the call_geolocator_latlon_updated()
### function to avoid violation of privacy.
library(dplyr, warn.conflicts = FALSE)
library(tigris)
library(lubridate)

# Input the geocoded addrsses csv file ("adrs_geocoded.csv")
adrs_geocoded <- read.csv("adrs_geocoded.csv")

# Perturb the last digit: if in 0-8, add 1; if 9, minus 1.
# Perturb longitudes
longitudes <- round(adrs_geocoded$lon, digits=4)
lon_char <- sprintf("%.4f", longitudes)

perturb <- function(lon_char) {
  if (lon_char == "NA") {return(lon_char)}
  lon <- strsplit(lon_char, "\\.")
  lon <- unlist(lon)
  int <- lon[1]
  dec <- lon[2]
  last <- substr(dec, 4, 4)
  if (last == "9") {new_last <- "8"} 
  else {new_last <- as.character(as.numeric(last) + 1)}
  dec <- paste0(substr(dec, 1, 3), new_last)
  new_lon_char <- paste0(int, ".", dec)
  return(new_lon_char)
}

# Use the function to lon_char
new_lon_char <- sapply(lon_char, perturb)
new_lon_char <- as.numeric(new_lon_char)

adrs_geocoded$lon_perturbed <- as.numeric(new_lon_char)

# Perturb latitudes
latitudes <- round(adrs_geocoded$lat, digits=4)
lat_char <- sprintf("%.4f", latitudes)

perturb <- function(lat_char) {
  if (lat_char == "NA") {return(lat_char)}
  lat <- strsplit(lat_char, "\\.")
  lat <- unlist(lat)
  int <- lat[1]
  dec <- lat[2]
  last <- substr(dec, 4, 4)
  if (last == "9") {new_last <- "8"} 
  else {new_last <- as.character(as.numeric(last) + 1)}
  dec <- paste0(substr(dec, 1, 3), new_last)
  new_lat_char <- paste0(int, ".", dec)
  return(new_lat_char)
}

# Use the function to lat_char
new_lat_char <- sapply(lat_char, perturb)
new_lat_char <- as.numeric(new_lat_char)

adrs_geocoded$lat_perturbed <- as.numeric(new_lat_char)

check <- adrs_geocoded %>% select(lon,lon_perturbed,lat,lat_perturbed)

# Generate FIPS codes at the census block level
# The general format of the name is DatasetType_SpatialBenchmark. 
#    DatasetType:Public_AR
# SpatialBenchmark: Current, ACS2023, Census2020
# The general format of the name is GeographyVintage_SpatialBenchmark. 
# Vintage list(Benchmark=Current): Current_Current, Census2010_Current, ACS2017_Current, ACS2018_Current, 
# ACS2019_Current, Census2020_Current, ACS2021_Current, ACS2022_Current, ACS2023_Current
# Benchmark=ACS2023: Similar list, just change the latter Benchmark to name to ACS2023
# Benchmark=Census2020: Census2020_Census2020, Census2010_Census2020



# code from author of the function call_geolocator_latlon to address updated in census data
# the new function is call_geolocator_latlon_updated()

call_geolocator_latlon_updated <- function(lat, lon, benchmark='Public_AR_Current', vintage='Current_Current') {
  # Build url
  call_start <- "https://geocoding.geo.census.gov/geocoder/geographies/coordinates?"
  
  url <- paste0("x=", lon,"&y=", lat)
  
  url_full <- paste0(call_start, url, "&benchmark=", benchmark, "&vintage=", vintage, "&format=json")
  
  # Check response
  r <- httr::GET(url_full)
  httr::stop_for_status(r)
  response <- httr::content(r)
  
  if (length(response$result$geographies) == 0) {
    message(paste0("Lat/lon (", lat, ", ", lon,
                   ") returned no geocodes. An NA was returned."))
    return(NA_character_)
  } else {
    
    #regex search for block group geography in response
    response_block<-grep(response[["result"]][["geographies"]], pattern = ".Block.")
    
    #check If a block group result is found or return NA
    #If block group response is found check GEOID length and return either NA for missing data or the value
    if(length(response_block) == 0){
      return(NA_character_)
    } else {
      if (length(response[["result"]][["geographies"]][[response_block]][[1]]$GEOID) == 0) {
        message(paste0("Lat/lon (", lat, ", ", lon,
                       ") returned no geocodes. An NA was returned."))
        return(NA_character_)
      } else {
        if (length(response[["result"]][["geographies"]][[response_block]][[1]]$GEOID) > 1) {
          message(paste0("Lat/lon (", lat, ", ", lon,
                         ") returned more than geocode. The first match was returned."))
        }
        
        return(response[["result"]][["geographies"]][[response_block]][[1]]$GEOID)
      }
    }
    
  }
}

#--------------------------------------------
# address before 2020
adrs_geocoded_2010 <- adrs_geocoded %>% filter(year(start)<2020)

# These are perturbed estimates to avoid violations of privacy
latitudes <- adrs_geocoded_2010$lat_perturbed
longitudes <- adrs_geocoded_2010$lon_perturbed
fips_block <- c()
for (i in 1:length(latitudes)) {  #latitudes and longitudes should have the same length, so we just take latitudes here
  if (!is.na(latitudes[i]) & !is.na(longitudes[i])) {
    current <- call_geolocator_latlon_updated(latitudes[i], longitudes[i], vintage = "Census2010_Current")
    fips_block <- c(fips_block, current)
  }
}

# The input address with NA latitude and longitutde is getting dropped
index <- which(is.na(adrs_geocoded_2010$lat)&is.na(adrs_geocoded_2010$lon))
sprintf("The following input addresses have been dropped due to lack of geocoded latitude and longitude: %d", index)

# Drop the addresses with missing value
adrs_geocoded_copy_2010 <- na.omit(adrs_geocoded_2010)

# FIPS code of state is the first 2 digits of the FIPS code of census block
fips_state<- substr(fips_block, 1, 2)

# FIPS code of county is the first 5 digits of the FIPS code of census block
fips_county <- substr(fips_block, 1, 5)

# FIPS code of census tract is the first 11 digits of the FIPS code of census block
fips_tract <- substr(fips_block, 1, 11)

# Save the FIPS codes at different geography level into a tibble
fips_alternative <-  tibble::tibble(
  cohortPatientId = adrs_geocoded_copy_2010$cohortPatientId,
  FIPS_State = fips_state,
  FIPS_County = fips_county,
  FIPS_Census_Tract = fips_tract,
  FIPS_Block = fips_block,
  ZIP = adrs_geocoded_copy_2010$ZIP,
  month = adrs_geocoded_copy_2010$month,
  day = adrs_geocoded_copy_2010$day,
  year = adrs_geocoded_copy_2010$year,
  start=adrs_geocoded_copy_2010$start,
  end=adrs_geocoded_copy_2010$end,
  record_date=adrs_geocoded_copy_2010$record_date
)

# Export the FIPS numbers to the csv file
write.csv(fips_alternative, file = "fips_nums_2010.csv", row.names = FALSE)



# encounter error message: Bad Gateway (HTTP 502).
# need to run again later
# the error did not show up the next day I run the code
#----------------------------------------------------
# address after 2020 (>=2020)
# Input the geocoded addrsses csv file ("adrs_geocoded.csv")
adrs_geocoded_2020 <- adrs_geocoded %>% filter(year(start)>=2020)

# These are perturbed estimates to avoid violations of privacy
latitudes <- adrs_geocoded_2020$lat_perturbed
longitudes <- adrs_geocoded_2020$lon_perturbed

# Generate FIPS codes at the census block level
# The general format of the name is DatasetType_SpatialBenchmark. 
#    DatasetType:Public_AR
# SpatialBenchmark: Current, ACS2023, Census2020
# The general format of the name is GeographyVintage_SpatialBenchmark. 
# Vintage list(Benchmark=Current): Current_Current, Census2010_Current, ACS2017_Current, ACS2018_Current, 
# ACS2019_Current, Census2020_Current, ACS2021_Current, ACS2022_Current, ACS2023_Current
# Benchmark=ACS2023: Similar list, just change the latter Benchmark to name to ACS2023
# Benchmark=Census2020: Census2020_Census2020, Census2010_Census2020
# for year on or after 2020, use the current census data
fips_block <- c()
for (i in 1:length(latitudes)) {  #latitudes and longitudes should have the same length, so we just take latitudes here
  if (!is.na(latitudes[i]) & !is.na(longitudes[i])) {
    current <- call_geolocator_latlon_updated(latitudes[i], longitudes[i])
    fips_block <- c(fips_block, current)
  }
}

# The input address with NA latitude and longitutde is getting dropped
index <- which(is.na(adrs_geocoded_2020$lat)&is.na(adrs_geocoded_2020$lon))
sprintf("The following input addresses have been dropped due to lack of geocoded latitude and longitude: %d", index)

# Drop the addresses with missing value
adrs_geocoded_copy_2020 <- na.omit(adrs_geocoded_2020)

# FIPS code of state is the first 2 digits of the FIPS code of census block
fips_state<- substr(fips_block, 1, 2)

# FIPS code of county is the first 5 digits of the FIPS code of census block
fips_county <- substr(fips_block, 1, 5)

# FIPS code of census tract is the first 11 digits of the FIPS code of census block
fips_tract <- substr(fips_block, 1, 11)

# Save the FIPS codes at different geography level into a tibble
fips_alternative <-  tibble::tibble(
  cohortPatientId = adrs_geocoded_copy_2020$cohortPatientId,
  FIPS_State = fips_state,
  FIPS_County = fips_county,
  FIPS_Census_Tract = fips_tract,
  FIPS_Block = fips_block,
  ZIP = adrs_geocoded_copy_2020$ZIP,
  month = adrs_geocoded_copy_2020$month,
  day = adrs_geocoded_copy_2020$day,
  year = adrs_geocoded_copy_2020$year,
  start=adrs_geocoded_copy_2020$start,
  end=adrs_geocoded_copy_2020$end,
  record_date=adrs_geocoded_copy_2020$record_date
)

# Export the FIPS numbers to the csv file
write.csv(fips_alternative, file = "fips_nums_2020.csv", row.names = FALSE)

#combine 2010 and 2020 fips nums
fips_nums_2010 <- read.csv("fips_nums_2010.csv") 
fips_nums_2020 <- read.csv("fips_nums_2020.csv") 
fips_nums <- bind_rows(fips_nums_2010, fips_nums_2020) %>% 
  arrange(cohortPatientId, start)
write.csv(fips_nums, file = "fips_nums.csv", row.names = FALSE)