## The GENERATE Tool is created by NA-ACCORD for local address geocoding and 
## submission of area-Level indicators

# This script aims to use the FIPS codes generated by the "fips_generator.R" script to pull the desired census data

# Load the necessary libraries
library(dplyr, quietly = TRUE)
library(tidycensus, quietly = TRUE)
library(tidyverse, quietly = TRUE)

# Import the geocoded addresses
adrs_geocoded <- read.csv("adrs_geocoded.csv") 

# Import the fips number generated
fips_nums <- read.csv("fips_nums.csv") 

# Remove the scientific notation format for the census block FIPS code
fips_nums$FIPS_Block <- format(fips_nums$FIPS_Block, scientific=F)

# Remove those with N/A fips numbers 
fips_nums_cleaned <- na.omit(fips_nums)
# Display the number of addresses getting dropped
num_dropped = nrow(fips_nums) - nrow(fips_nums_cleaned)
sprintf("The number of dropped addresses is: %d", num_dropped)

# Because csv files automatically remove inital zeros, we will have to refill them here
state_fips <- sprintf("%02s", fips_nums_cleaned$FIPS_State)
fips_nums_cleaned$FIPS_State <- state_fips
county_fips <- sprintf("%05s", fips_nums_cleaned$FIPS_County)
fips_nums_cleaned$FIPS_County <- county_fips
tract_fips <- sprintf("%011s", fips_nums_cleaned$FIPS_Census_Tract)
fips_nums_cleaned$FIPS_Census_Tract <- tract_fips
zip_codes <- sprintf("%05s", fips_nums_cleaned$ZIP)
fips_nums_cleaned$ZIP <- zip_codes

# Check the session info 
sessionInfo()

# Set census key
census_api_key("7120d9f7b3f5e2fdfe751c5cf918295f9e032aff", install=TRUE, overwrite=TRUE)
readRenviron("~/.Renviron")
Sys.getenv("CENSUS_API_KEY")


# To obtain the ACS estimates at census tract level, the state of the census tract desired needs to be specified in the get_acs() function
# Hence, we will loop through the unique states of the geocoded input addresses ("adrs_geocoded" variable), and extract ACS estimates for each specific state

# Obtain the unique states for the geocoded input addresses as well as the number for it
uni_states = unique(fips_nums_cleaned$FIPS_State)  #Unique states
uni_state_num = length(uni_states)  #Number of unique states

# We have three types of estimates required: 
# median household income, proportion of employed, proportion of having associate degrees or above

# The years we are getting census data from are 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021.
# We can create a vector that stored these years
# year_list = c(2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021)
# But if not specified, then we extract the census data according to which years were in "adrs_geocoded.csv"
year_list = sort(unique(adrs_geocoded$year))


#########################################
#### Median household income - State ####
#########################################
# Create a function to get the required census estimates for median income at the state level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_inc_state <- function(year, survey) {  
  # Initialize an empty tibble
  income_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    income <- get_acs(geography = "state",
                      variables = c(median_income = "B19013_001"),
                      survey = survey,
                      output = "tidy",
                      state = uni_states[i],
                      year = year
    )
    # Append the income data to the existing tibble
    income_data <- bind_rows(income_data, income)
  }
  # Return the final tibble with all income data
  return(income_data)
}


# Initialize an empty tibble
median_inc_state <- tibble()
# Loop through these years and append the median household income estimates of each year into one tibble
for (j in year_list) {
  income_year <- pull_inc_state(j, "acs5") # For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in income_year tibble
  income_year <- income_year %>% mutate(year = j)
  
  # Append the income data to the existing tibble
  median_inc_state <- bind_rows(median_inc_state, income_year)
}


# Rename the GEOID in median_inc to "FIPS_Census_Tract"
median_inc_state <- median_inc_state %>% rename(FIPS_State = GEOID)

# Join the median_inc and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
median_income_state <- inner_join(median_inc_state, fips_nums_cleaned, by = c("FIPS_State", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# median_income_state <- inner_join(median_inc_state, fips_nums_cleaned, by = "FIPS_State") %>%
#                                                 select(id, variable, estimate, year)



##########################################
#### Median household income - County ####
##########################################
# Create a function to get the required census estimates for median income at the county level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_inc_county <- function(year, survey) {  
  # Initialize an empty tibble
  income_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    income <- get_acs(geography = "county",
                      variables = c(median_income = "B19013_001"),
                      survey = survey,
                      output = "tidy",
                      state = uni_states[i],
                      year = year
    )
    # Append the income data to the existing tibble
    income_data <- bind_rows(income_data, income)
  }
  # Return the final tibble with all income data
  return(income_data)
}


# Initialize an empty tibble
median_inc_county <- tibble()
# Loop through these years and append the median household income estimates of each year into one tibble
for (j in year_list) {
  income_year <- pull_inc_county(j, "acs5") # For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in income_year tibble
  income_year <- income_year %>% mutate(year = j)
  
  # Append the income data to the existing tibble
  median_inc_county <- bind_rows(median_inc_county, income_year)
}


# Rename the GEOID in median_inc to "FIPS_Census_Tract"
median_inc_county <- median_inc_county %>% rename(FIPS_County = GEOID)

# Join the median_inc and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
median_income_county <- inner_join(median_inc_county, fips_nums_cleaned, by = c("FIPS_County", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# median_income_county <- inner_join(median_inc_county, fips_nums_cleaned, by = "FIPS_County") %>%
#                                                 select(id, variable, estimate, year)



################################################
#### Median household income - Census tract ####
################################################
# Create a function to get the required census estimates for median income at the census tract level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_inc_tract <- function(year, survey) {  
  # Initialize an empty tibble
  income_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    income <- get_acs(geography = "tract",
                      variables = c(median_income = "B19013_001"),
                      survey = survey,
                      output = "tidy",
                      state = uni_states[i],
                      year = year
    )
    # Append the income data to the existing tibble
    income_data <- bind_rows(income_data, income)
  }
  # Return the final tibble with all income data
  return(income_data)
}


# Initialize an empty tibble
median_inc_tract <- tibble()
# Loop through these years and append the median household income estimates of each year into one tibble
for (j in year_list) {
  income_year <- pull_inc_tract(j, "acs5") # For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in income_year tibble
  income_year <- income_year %>% mutate(year = j)
  
  # Append the income data to the existing tibble
  median_inc_tract <- bind_rows(median_inc_tract, income_year)
}


# Rename the GEOID in median_inc to "FIPS_Census_Tract"
median_inc_tract <- median_inc_tract %>% rename(FIPS_Census_Tract = GEOID)

# Join the median_inc and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
median_income_tract <- inner_join(median_inc_tract, fips_nums_cleaned, by = c("FIPS_Census_Tract", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# median_income_tract <- inner_join(median_inc_tract, fips_nums_cleaned, by = "FIPS_Census_Tract") %>%
#                                                 select(id, variable, estimate, year)



########################################
#### Median household income - ZCTA ####
########################################
# Create a function to get the required census estimates for median income at ZCTA level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_inc_zcta <- function(year, survey) {  
  # Initialize an empty tibble
  income_data <- tibble()
  
  # Pull the census data and assign them to the income_data tibble
  income_data <- get_acs(geography = "zcta",
                    variables = c(median_income = "B19013_001"),
                    survey = survey,
                    output = "tidy",
                    year = year
  )

  # Return the final tibble with all income data
  return(income_data)
}


# Initialize an empty tibble
median_inc_zcta <- tibble()
# Loop through these years and append the median household income estimates of each year into one tibble
for (j in year_list) {
  income_year <- pull_inc_zcta(j, "acs5") # For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in income_year tibble
  income_year <- income_year %>% mutate(year = j)
  
  # Append the income data to the existing tibble
  median_inc_zcta <- bind_rows(median_inc_zcta, income_year)
}


# Rename the GEOID in median_inc to "FIPS_Census_Tract"
median_inc_zcta <- median_inc_zcta %>% rename(ZIP = GEOID)

# For year 2012, because the ZIP codes are different where they have 7 digits and only the last 5 digits are actually the ZIPs, we will extract them
# This does not seem to occur from year 2013-2021, but if future years have the same situation as year 2012, same thing needs to be done
zips_inc_2012 <- substr(median_inc_zcta$ZIP,3,7)
median_inc_zcta <- median_inc_zcta %>% 
                    mutate(ZIP = ifelse(year==2012, zips_inc_2012, ZIP))

# Extract the ZCTAs
# zcta <- substr(median_inc_zcta$NAME,7,11)


# Join the median_inc and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
median_income_zcta <- inner_join(median_inc_zcta, fips_nums_cleaned, by = c("ZIP", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# median_income_zcta <- inner_join(median_inc_zcta, fips_nums_cleaned, by = "ZIP") %>%
#                                                 select(id, variable, estimate, year)




##########################################
#### Proportion of employment - State ####
##########################################

# We need separate functions for number of people in labor force and number of civilian labor force, because it is easier to retrieve the data

# Create a function to get the required census estimates at the state level for number of people in labor force (the denominator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_inlabor_state <- function(year, survey) {  
  # Initialize an empty tibble
  inlabor_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    inlabor <- get_acs(geography = "state",
                       variables = c(in_labor = "B23025_003"),
                       survey = survey,
                       output = "tidy",
                       state = uni_states[i],
                       year = year,
                       moe_level = 95
    )
    # Append the employment data to the existing tibble
    inlabor_data <- bind_rows(inlabor_data, inlabor)
  }
  # Return the final tibble with all in labor force data
  return(inlabor_data)
}


# Create a function to get the required census estimates at the state level for number of civilian labor force (the numerator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_civilian_state <- function(year, survey) {  
  # Initialize an empty tibble
  civilian_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    civilian <- get_acs(geography = "state",
                        variables = c(civilian_labor = "B23025_004"),
                        survey = survey,
                        output = "tidy",
                        state = uni_states[i],
                        year = year,
                        moe_level = 95
    )
    # Append the employment data to the existing tibble
    civilian_data <- bind_rows(civilian_data, civilian)
  }
  # Return the final tibble with all civilian labor data
  return(civilian_data)
}


# Initialize two empty tibbles to store the number of in labor (denominator) and number of civilian labor (numerator) respectively
inlabor_state <- tibble()
civilian_state <- tibble()
# Loop through these years and append the number of in labor force and the number of civilian labor of each year into two respective tibble
for (j in year_list) {
  # Create two temporary tibbles to store the census data for this specific iteration (year)
  inlabor_cur <- pull_inlabor_state(j, "acs5")    
  civilian_cur <- pull_civilian_state(j, "acs5")   #For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in the inlabor_cur tibble and the civilian_cur tibble
  inlabor_cur <- inlabor_cur %>% mutate(year = j)
  civilian_cur <- civilian_cur %>% mutate(year = j)
  
  # Append the temporary in-labor data and civilian-labor data to the existing tibble
  inlabor_state <- bind_rows(inlabor_state, inlabor_cur)
  civilian_state <- bind_rows(civilian_state, civilian_cur)
}

# A check for whether the GEOID has the same values for inlabor tibble and civilian tibble
identical(inlabor_state$GEOID,civilian_state$GEOID)

# Create another tibble for proportion of employment (number of civilian labor / number in labor force)
# It doesn't matter to use whether "inlabor" or "civilian"'s GEOID or NAME because their GEOID & NAME is the same
employ_state <- tibble (FIPS_State = inlabor_state$GEOID,
                        NAME = inlabor_state$NAME,
                        variable = "employ_prop",
                        estimate = (civilian_state$estimate / inlabor_state$estimate),
                        year = inlabor_state$year
)

# Join the employ and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
employment_state <- inner_join(employ_state, fips_nums_cleaned, by = c("FIPS_State", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# employment_state <- inner_join(employ_state, fips_nums_cleaned, by = "FIPS_State") %>%
#                                                 select(cohortPatientId, variable, estimate, year)


# Sanity check function (the estimates for employment should not be greater than 1, because it is a proportion)
if (!all(employment_state$estimate <= 1, na.rm=TRUE)) {
  print("Employment proportion should not be greater than 1")
} else {
  print("All employment proportions are smaller or equal to 1")
}




###########################################
#### Proportion of employment - County ####
###########################################

# We need separate functions for number of people in labor force and number of civilian labor force, because it is easier to retrieve the data

# Create a function to get the required census estimates at county level for number of people in labor force (the denominator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_inlabor_county <- function(year, survey) {  
  # Initialize an empty tibble
  inlabor_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    inlabor <- get_acs(geography = "county",
                       variables = c(in_labor = "B23025_003"),
                       survey = survey,
                       output = "tidy",
                       state = uni_states[i],
                       year = year,
                       moe_level = 95
    )
    # Append the employment data to the existing tibble
    inlabor_data <- bind_rows(inlabor_data, inlabor)
  }
  # Return the final tibble with all in labor force data
  return(inlabor_data)
}


# Create a function to get the required census estimates at county level for number of civilian labor force (the numerator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_civilian_county <- function(year, survey) {  
  # Initialize an empty tibble
  civilian_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    civilian <- get_acs(geography = "county",
                        variables = c(civilian_labor = "B23025_004"),
                        survey = survey,
                        output = "tidy",
                        state = uni_states[i],
                        year = year,
                        moe_level = 95
    )
    # Append the employment data to the existing tibble
    civilian_data <- bind_rows(civilian_data, civilian)
  }
  # Return the final tibble with all civilian labor data
  return(civilian_data)
}


# Initialize two empty tibbles to store the number of in labor (denominator) and number of civilian labor (numerator) respectively
inlabor_county <- tibble()
civilian_county <- tibble()
# Loop through these years and append the number of in labor force and the number of civilian labor of each year into two respective tibble
for (j in year_list) {
  # Create two temporary tibbles to store the census data for this specific iteration (year)
  inlabor_cur <- pull_inlabor_county(j, "acs5")    
  civilian_cur <- pull_civilian_county(j, "acs5")   #For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in the inlabor_cur tibble and the civilian_cur tibble
  inlabor_cur <- inlabor_cur %>% mutate(year = j)
  civilian_cur <- civilian_cur %>% mutate(year = j)
  
  # Append the temporary in-labor data and civilian-labor data to the existing tibble
  inlabor_county <- bind_rows(inlabor_county, inlabor_cur)
  civilian_county <- bind_rows(civilian_county, civilian_cur)
}

# A check for whether the GEOID has the same values for inlabor tibble and civilian tibble
identical(inlabor_county$GEOID,civilian_county$GEOID)

# Create another tibble for proportion of employment (number of civilian labor / number in labor force)
# It doesn't matter to use whether "inlabor" or "civilian"'s GEOID or NAME because their GEOID & NAME is the same
employ_county <- tibble (FIPS_County = inlabor_county$GEOID,
                  NAME = inlabor_county$NAME,
                  variable = "employ_prop",
                  estimate = (civilian_county$estimate / inlabor_county$estimate),
                  year = inlabor_county$year
)

# Join the employ and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
employment_county <- inner_join(employ_county, fips_nums_cleaned, by = c("FIPS_County", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# employment_county <- inner_join(employ_county, fips_nums_cleaned, by = "FIPS_County") %>%
#                                                 select(cohortPatientId, variable, estimate, year)


# Sanity check function (the estimates for employment should not be greater than 1, because it is a proportion)
if (!all(employment_county$estimate <= 1, na.rm=TRUE)) {
  print("Employment proportion should not be greater than 1")
} else {
  print("All employment proportions are smaller or equal to 1")
}




#################################################
#### Proportion of employment - Census tract ####
#################################################

# We need separate functions for number of people in labor force and number of civilian labor force, because it is easier to retrieve the data

# Create a function to get the required census estimates at the census tract level for number of people in labor force (the denominator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_inlabor_tract <- function(year, survey) {  
  # Initialize an empty tibble
  inlabor_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    inlabor <- get_acs(geography = "tract",
                       variables = c(in_labor = "B23025_003"),
                       survey = survey,
                       output = "tidy",
                       state = uni_states[i],
                       year = year,
                       moe_level = 95
    )
    # Append the employment data to the existing tibble
    inlabor_data <- bind_rows(inlabor_data, inlabor)
  }
  # Return the final tibble with all in labor force data
  return(inlabor_data)
}


# Create a function to get the required census estimates at the census tract level for number of civilian labor force (the numerator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_civilian_tract <- function(year, survey) {  
  # Initialize an empty tibble
  civilian_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    civilian <- get_acs(geography = "tract",
                        variables = c(civilian_labor = "B23025_004"),
                        survey = survey,
                        output = "tidy",
                        state = uni_states[i],
                        year = year,
                        moe_level = 95
    )
    # Append the employment data to the existing tibble
    civilian_data <- bind_rows(civilian_data, civilian)
  }
  # Return the final tibble with all civilian labor data
  return(civilian_data)
}


# Initialize two empty tibbles to store the number of in labor (denominator) and number of civilian labor (numerator) respectively
inlabor_tract <- tibble()
civilian_tract <- tibble()
# Loop through these years and append the number of in labor force and the number of civilian labor of each year into two respective tibble
for (j in year_list) {
  # Create two temporary tibbles to store the census data for this specific iteration (year)
  inlabor_cur <- pull_inlabor_tract(j, "acs5")    
  civilian_cur <- pull_civilian_tract(j, "acs5")   #For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in the inlabor_cur tibble and the civilian_cur tibble
  inlabor_cur <- inlabor_cur %>% mutate(year = j)
  civilian_cur <- civilian_cur %>% mutate(year = j)
  
  # Append the temporary in-labor data and civilian-labor data to the existing tibble
  inlabor_tract <- bind_rows(inlabor_tract, inlabor_cur)
  civilian_tract <- bind_rows(civilian_tract, civilian_cur)
}

# A check for whether the GEOID has the same values for inlabor tibble and civilian tibble
identical(inlabor_tract$GEOID,civilian_tract$GEOID)

# Create another tibble for proportion of employment (number of civilian labor / number in labor force)
# It doesn't matter to use whether "inlabor" or "civilian"'s GEOID or NAME because their GEOID & NAME is the same
employ_tract <- tibble (FIPS_Census_Tract = inlabor_tract$GEOID,
                  NAME = inlabor_tract$NAME,
                  variable = "employ_prop",
                  estimate = (civilian_tract$estimate / inlabor_tract$estimate),
                  year = inlabor_tract$year
)

# Join the employ and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
employment_tract <- inner_join(employ_tract, fips_nums_cleaned, by = c("FIPS_Census_Tract", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# employment <- inner_join(employ, fips_nums_cleaned, by = "FIPS_Census_Tract") %>%
#                                                 select(cohortPatientId, variable, estimate, year)


# Sanity check function (the estimates for employment should not be greater than 1, because it is a proportion)
if (!all(employment_tract$estimate <= 1, na.rm=TRUE)) {
  print("Employment proportion should not be greater than 1")
} else {
  print("All employment proportions are smaller or equal to 1")
}



#########################################
#### Proportion of employment - ZCTA ####
#########################################

# We need separate functions for number of people in labor force and number of civilian labor force, because it is easier to retrieve the data

# Create a function to get the required census estimates at the ZCTA level for number of people in labor force (the denominator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_inlabor_zcta <- function(year, survey) {  
  # Initialize an empty tibble
  inlabor_data <- tibble()
  inlabor_data <- get_acs(geography = "zcta",
                     variables = c(in_labor = "B23025_003"),
                     survey = survey,
                     output = "tidy",
                     year = year,
                     moe_level = 95
    )
   
  # Return the final tibble with all in labor force data
  return(inlabor_data)
}


# Create a function to get the required census estimates at the ZCTA level for number of civilian labor force (the numerator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_civilian_zcta <- function(year, survey) {  
  # Initialize an empty tibble
  civilian_data <- tibble()
  
  civilian_data <- get_acs(geography = "zcta",
                        variables = c(civilian_labor = "B23025_004"),
                        survey = survey,
                        output = "tidy",
                        year = year,
                        moe_level = 95
    )

  # Return the final tibble with all civilian labor data
  return(civilian_data)
}


# Initialize two empty tibbles to store the number of in labor (denominator) and number of civilian labor (numerator) respectively
inlabor_zcta<- tibble()
civilian_zcta <- tibble()
# Loop through these years and append the number of in labor force and the number of civilian labor of each year into two respective tibble
for (j in year_list) {
  # Create two temporary tibbles to store the census data for this specific iteration (year)
  inlabor_cur <- pull_inlabor_zcta(j, "acs5")    
  civilian_cur <- pull_civilian_zcta(j, "acs5")   #For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in the inlabor_cur tibble and the civilian_cur tibble
  inlabor_cur <- inlabor_cur %>% mutate(year = j)
  civilian_cur <- civilian_cur %>% mutate(year = j)
  
  # Append the temporary in-labor data and civilian-labor data to the existing tibble
  inlabor_zcta <- bind_rows(inlabor_zcta, inlabor_cur)
  civilian_zcta <- bind_rows(civilian_zcta, civilian_cur)
}

# A check for whether the GEOID has the same values for inlabor tibble and civilian tibble
identical(inlabor_zcta$GEOID,civilian_zcta$GEOID)

# Create another tibble for proportion of employment (number of civilian labor / number in labor force)
# It doesn't matter to use whether "inlabor" or "civilian"'s GEOID or NAME because their GEOID & NAME is the same
employ_zcta <- tibble (ZIP = inlabor_zcta$GEOID,
                        NAME = inlabor_zcta$NAME,
                        variable = "employ_prop",
                        estimate = (civilian_zcta$estimate / inlabor_zcta$estimate),
                        year = inlabor_zcta$year
)

# For year 2012, because the ZIP codes are different where they have 7 digits and only the last 5 digits are actually the ZIPs, we will extract them
# This does not seem to occur from year 2013-2021, but if future years have the same situation as year 2012, same thing needs to be done
zips_emp_2012 <- substr(employ_zcta$ZIP,3,7)
employ_zcta <- employ_zcta %>% 
  mutate(ZIP = ifelse(year==2012, zips_emp_2012, ZIP))

# Extract the ZCTAs
# zcta <- substr(employ_zcta$NAME,7,11)

# Join the employ and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
employment_zcta <- inner_join(employ_zcta, fips_nums_cleaned, by = c("ZIP", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# employment_zcta <- inner_join(employ_zcta, fips_nums_cleaned, by = "ZIP") %>%
#                                                 select(cohortPatientId, variable, estimate, year)


# Sanity check function (the estimates for employment should not be greater than 1, because it is a proportion)
if (!all(employment_zcta$estimate <= 1, na.rm=TRUE)) {
  print("Employment proportion should not be greater than 1")
} else {
  print("All employment proportions are smaller or equal to 1")
}



#########################################
#### Proportion of education - State ####
#########################################

# We need separate functions for total (denominator), 
# the number of people who received associate's degree, 
# the number of people who received Bachelor's degree,
# the number of people who received Master's degree,
# the number of people who received Professional degree,
# the number of people who received Doctorate degree


# Create a function to get the required census estimates at the state level for the total (denominator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_total_state <- function(year, survey) {  
  # Initialize an empty tibble
  total_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    total <- get_acs(geography = "state",
                     variables = c(total = "B15003_001"),
                     survey = survey,
                     output = "tidy",
                     state = uni_states[i],
                     year = year,
                     moe_level = 95
    )
    # Append the total data to the existing tibble
    total_data <- bind_rows(total_data, total)
  }
  # Return the final tibble with all total data
  return(total_data)
}


# Create a function to get the number who received associate's degree at the state level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_associate_state <- function(year, survey) {  
  # Initialize an empty tibble
  associate_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    associate <- get_acs(geography = "state",
                         variables = c(associate = "B15003_021"),
                         survey = survey,
                         output = "tidy",
                         state = uni_states[i],
                         year = year,
                         moe_level = 95
    )
    # Append the associate's degree data to the existing tibble
    associate_data <- bind_rows(associate_data, associate)
  }
  # Return the final tibble with all associate's degree data
  return(associate_data)
}


# Create a function to get the number who received Bachelor's degree at the state level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_bachelor_state <- function(year, survey) {  
  # Initialize an empty tibble
  bachelor_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    bachelor <- get_acs(geography = "state",
                        variables = c(bachelor = "B15003_022"),
                        survey = survey,
                        output = "tidy",
                        state = uni_states[i],
                        year = year,
                        moe_level = 95
    )
    # Append the Bachelor's degree data to the existing tibble
    bachelor_data <- bind_rows(bachelor_data, bachelor)
  }
  # Return the final tibble with all Bachelor's degree data
  return(bachelor_data)
}


# Create a function to get the number who received Master's degree at the state level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_master_state <- function(year, survey) {  
  # Initialize an empty tibble
  master_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    master <- get_acs(geography = "state",
                      variables = c(master = "B15003_023"),
                      survey = survey,
                      output = "tidy",
                      state = uni_states[i],
                      year = year,
                      moe_level = 95
    )
    # Append the Master's degree data to the existing tibble
    master_data <- bind_rows(master_data, master)
  }
  # Return the final tibble with all Master's degree data
  return(master_data)
}


# Create a function to get the number who received professional degree at the state level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_profess_state <- function(year, survey) {  
  # Initialize an empty tibble
  profess_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    profession <- get_acs(geography = "state",
                          variables = c(professional = "B15003_024"),
                          survey = survey,
                          output = "tidy",
                          state = uni_states[i],
                          year = year,
                          moe_level = 95
    )
    # Append the professional degree data to the existing tibble
    profess_data <- bind_rows(profess_data, profession)
  }
  # Return the final tibble with all professional degree data
  return(profess_data)
}


# Create a function to get the number who received Doctorate degree at the state level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_doctorate_state <- function(year, survey) {  
  # Initialize an empty tibble
  doc_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    doctorate <- get_acs(geography = "state",
                         variables = c(doctorate = "B15003_025"),
                         survey = survey,
                         output = "tidy",
                         state = uni_states[i],
                         year = year,
                         moe_level = 95
    )
    # Append the Doctorate degree data to the existing tibble
    doc_data <- bind_rows(doc_data, doctorate)
  }
  # Return the final tibble with all Doctorate's degree data
  return(doc_data)
}


# Initialize the empty tibbles to store the total number (denominator) and the respective number of people with the following degrees:
# associate's degree, bachelor's degree, master's degree, professional degree, doctorate degree
total_state <- tibble()
associate_state <- tibble()
bachelor_state <- tibble()
master_state <- tibble()
profession_state <- tibble()
doctorate_state <- tibble()
# Loop through these years and append the number of total educated and each education category to their respective tibble
for (j in year_list) {
  # Create two temporary tibbles to store the census data for this specific iteration (year)
  total_cur <- pull_total_state(j, "acs5")
  associate_cur <- pull_associate_state(j, "acs5")
  bachelor_cur <- pull_bachelor_state(j, "acs5")   
  master_cur <- pull_master_state(j, "acs5")
  profess_cur <- pull_profess_state(j, "acs5")   
  doctorate_cur <- pull_doctorate_state(j, "acs5")   #For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in the total_cur tibble and the educate_cur tibble
  total_cur <- total_cur %>% mutate(year = j)
  associate_cur <- associate_cur %>% mutate(year = j)
  bachelor_cur <- bachelor_cur %>% mutate(year = j)
  master_cur <- master_cur %>% mutate(year = j)
  profess_cur <- profess_cur %>% mutate(year = j)
  doctorate_cur <- doctorate_cur %>% mutate(year = j)
  
  # Append the temporary data and educate data to the existing tibble
  total_state <- bind_rows(total_state, total_cur)
  associate_state <- bind_rows(associate_state, associate_cur)
  bachelor_state <- bind_rows(bachelor_state, bachelor_cur)
  master_state <- bind_rows(master_state, master_cur)
  profession_state <- bind_rows(profession_state, profess_cur)
  doctorate_state <- bind_rows(doctorate_state, doctorate_cur)
}


# A check for whether the GEOID has the same values for all these tibbles
identical(total_state$GEOID, associate_state$GEOID)
identical(total_state$GEOID, bachelor_state$GEOID)
identical(total_state$GEOID, master_state$GEOID)
identical(total_state$GEOID, profession_state$GEOID)
identical(total_state$GEOID, doctorate_state$GEOID)

# Create another tibble for proportion of education (sum of numbers of each education degrees / total)
educate_state <- tibble (FIPS_State = total_state$GEOID,
                          NAME = total_state$NAME,
                          variable = "education_prop",
                          estimate = ((associate_state$estimate + bachelor_state$estimate + master_state$estimate + profession_state$estimate + doctorate_state$estimate) / total_state$estimate),
                          year = total_state$year
)


# Join the educate and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
education_state <- inner_join(educate_state, fips_nums_cleaned, by = c("FIPS_State", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# education_county <- inner_join(educate_county, fips_nums_cleaned, by = "FIPS_County") %>%
#                                                 select(cohortPatientId, variable, estimate, year)


# Sanity check function (the estimates for education should not be greater than 1, because it is a proportion)
if (!all(education_state$estimate <= 1, na.rm=TRUE)) {
  print("Education proportion should not be greater than 1")
} else {
  print("All education proportions are smaller or equal to 1")
}




##########################################
#### Proportion of education - County ####
##########################################

# We need separate functions for total (denominator), 
# the number of people who received associate's degree, 
# the number of people who received Bachelor's degree,
# the number of people who received Master's degree,
# the number of people who received Professional degree,
# the number of people who received Doctorate degree


# Create a function to get the required census estimates at the county level for the total (denominator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_total_county <- function(year, survey) {  
  # Initialize an empty tibble
  total_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    total <- get_acs(geography = "county",
                     variables = c(total = "B15003_001"),
                     survey = survey,
                     output = "tidy",
                     state = uni_states[i],
                     year = year,
                     moe_level = 95
    )
    # Append the total data to the existing tibble
    total_data <- bind_rows(total_data, total)
  }
  # Return the final tibble with all total data
  return(total_data)
}


# Create a function to get the number who received associate's degree at the county level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_associate_county <- function(year, survey) {  
  # Initialize an empty tibble
  associate_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    associate <- get_acs(geography = "county",
                         variables = c(associate = "B15003_021"),
                         survey = survey,
                         output = "tidy",
                         state = uni_states[i],
                         year = year,
                         moe_level = 95
    )
    # Append the associate's degree data to the existing tibble
    associate_data <- bind_rows(associate_data, associate)
  }
  # Return the final tibble with all associate's degree data
  return(associate_data)
}


# Create a function to get the number who received Bachelor's degree at the county level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_bachelor_county <- function(year, survey) {  
  # Initialize an empty tibble
  bachelor_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    bachelor <- get_acs(geography = "county",
                        variables = c(bachelor = "B15003_022"),
                        survey = survey,
                        output = "tidy",
                        state = uni_states[i],
                        year = year,
                        moe_level = 95
    )
    # Append the Bachelor's degree data to the existing tibble
    bachelor_data <- bind_rows(bachelor_data, bachelor)
  }
  # Return the final tibble with all Bachelor's degree data
  return(bachelor_data)
}


# Create a function to get the number who received Master's degree at the county level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_master_county <- function(year, survey) {  
  # Initialize an empty tibble
  master_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    master <- get_acs(geography = "county",
                      variables = c(master = "B15003_023"),
                      survey = survey,
                      output = "tidy",
                      state = uni_states[i],
                      year = year,
                      moe_level = 95
    )
    # Append the Master's degree data to the existing tibble
    master_data <- bind_rows(master_data, master)
  }
  # Return the final tibble with all Master's degree data
  return(master_data)
}


# Create a function to get the number who received professional degree at the county level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_profess_county <- function(year, survey) {  
  # Initialize an empty tibble
  profess_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    profession <- get_acs(geography = "county",
                          variables = c(professional = "B15003_024"),
                          survey = survey,
                          output = "tidy",
                          state = uni_states[i],
                          year = year,
                          moe_level = 95
    )
    # Append the professional degree data to the existing tibble
    profess_data <- bind_rows(profess_data, profession)
  }
  # Return the final tibble with all professional degree data
  return(profess_data)
}


# Create a function to get the number who received Doctorate degree at the county level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_doctorate_county <- function(year, survey) {  
  # Initialize an empty tibble
  doc_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    doctorate <- get_acs(geography = "county",
                         variables = c(doctorate = "B15003_025"),
                         survey = survey,
                         output = "tidy",
                         state = uni_states[i],
                         year = year,
                         moe_level = 95
    )
    # Append the Doctorate degree data to the existing tibble
    doc_data <- bind_rows(doc_data, doctorate)
  }
  # Return the final tibble with all Doctorate's degree data
  return(doc_data)
}


# Initialize the empty tibbles to store the total number (denominator) and the respective number of people with the following degrees:
# associate's degree, bachelor's degree, master's degree, professional degree, doctorate degree
total_county <- tibble()
associate_county <- tibble()
bachelor_county <- tibble()
master_county <- tibble()
profession_county <- tibble()
doctorate_county <- tibble()
# Loop through these years and append the number of total educated and each education category to their respective tibble
for (j in year_list) {
  # Create two temporary tibbles to store the census data for this specific iteration (year)
  total_cur <- pull_total_county(j, "acs5")
  associate_cur <- pull_associate_county(j, "acs5")
  bachelor_cur <- pull_bachelor_county(j, "acs5")   
  master_cur <- pull_master_county(j, "acs5")
  profess_cur <- pull_profess_county(j, "acs5")   
  doctorate_cur <- pull_doctorate_county(j, "acs5")   #For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in the total_cur tibble and the educate_cur tibble
  total_cur <- total_cur %>% mutate(year = j)
  associate_cur <- associate_cur %>% mutate(year = j)
  bachelor_cur <- bachelor_cur %>% mutate(year = j)
  master_cur <- master_cur %>% mutate(year = j)
  profess_cur <- profess_cur %>% mutate(year = j)
  doctorate_cur <- doctorate_cur %>% mutate(year = j)
  
  # Append the temporary data and educate data to the existing tibble
  total_county <- bind_rows(total_county, total_cur)
  associate_county <- bind_rows(associate_county, associate_cur)
  bachelor_county <- bind_rows(bachelor_county, bachelor_cur)
  master_county <- bind_rows(master_county, master_cur)
  profession_county <- bind_rows(profession_county, profess_cur)
  doctorate_county <- bind_rows(doctorate_county, doctorate_cur)
}


# A check for whether the GEOID has the same values for all these tibbles
identical(total_county$GEOID, associate_county$GEOID)
identical(total_county$GEOID, bachelor_county$GEOID)
identical(total_county$GEOID, master_county$GEOID)
identical(total_county$GEOID, profession_county$GEOID)
identical(total_county$GEOID, doctorate_county$GEOID)

# Create another tibble for proportion of education (sum of numbers of each education degrees / total)
educate_county <- tibble (FIPS_County = total_county$GEOID,
                   NAME = total_county$NAME,
                   variable = "education_prop",
                   estimate = ((associate_county$estimate + bachelor_county$estimate + master_county$estimate + profession_county$estimate + doctorate_county$estimate) / total_county$estimate),
                   year = total_county$year
)


# Join the educate and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
education_county <- inner_join(educate_county, fips_nums_cleaned, by = c("FIPS_County", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# education_county <- inner_join(educate_county, fips_nums_cleaned, by = "FIPS_County") %>%
#                                                 select(cohortPatientId, variable, estimate, year)


# Sanity check function (the estimates for education should not be greater than 1, because it is a proportion)
if (!all(education_county$estimate <= 1, na.rm=TRUE)) {
  print("Education proportion should not be greater than 1")
} else {
  print("All education proportions are smaller or equal to 1")
}



################################################
#### Proportion of education - Census tract ####
################################################

# We need separate functions for total (denominator), 
# the number of people who received associate's degree, 
# the number of people who received Bachelor's degree,
# the number of people who received Master's degree,
# the number of people who received Professional degree,
# the number of people who received Doctorate degree


# Create a function to get the required census estimates at the census tract level for the total (denominator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_total_tract <- function(year, survey) {  
  # Initialize an empty tibble
  total_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    total <- get_acs(geography = "tract",
                     variables = c(total = "B15003_001"),
                     survey = survey,
                     output = "tidy",
                     state = uni_states[i],
                     year = year,
                     moe_level = 95
    )
    # Append the total data to the existing tibble
    total_data <- bind_rows(total_data, total)
  }
  # Return the final tibble with all total data
  return(total_data)
}


# Create a function to get the number who received associate's degree at the census tract level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_associate_tract <- function(year, survey) {  
  # Initialize an empty tibble
  associate_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    associate <- get_acs(geography = "tract",
                         variables = c(associate = "B15003_021"),
                         survey = survey,
                         output = "tidy",
                         state = uni_states[i],
                         year = year,
                         moe_level = 95
    )
    # Append the associate's degree data to the existing tibble
    associate_data <- bind_rows(associate_data, associate)
  }
  # Return the final tibble with all associate's degree data
  return(associate_data)
}


# Create a function to get the number who received Bachelor's degree at the census tract level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_bachelor_tract <- function(year, survey) {  
  # Initialize an empty tibble
  bachelor_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    bachelor <- get_acs(geography = "tract",
                        variables = c(bachelor = "B15003_022"),
                        survey = survey,
                        output = "tidy",
                        state = uni_states[i],
                        year = year,
                        moe_level = 95
    )
    # Append the Bachelor's degree data to the existing tibble
    bachelor_data <- bind_rows(bachelor_data, bachelor)
  }
  # Return the final tibble with all Bachelor's degree data
  return(bachelor_data)
}


# Create a function to get the number who received Master's degree at the census tract level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_master_tract <- function(year, survey) {  
  # Initialize an empty tibble
  master_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    master <- get_acs(geography = "tract",
                      variables = c(master = "B15003_023"),
                      survey = survey,
                      output = "tidy",
                      state = uni_states[i],
                      year = year,
                      moe_level = 95
    )
    # Append the Master's degree data to the existing tibble
    master_data <- bind_rows(master_data, master)
  }
  # Return the final tibble with all Master's degree data
  return(master_data)
}


# Create a function to get the number who received professional degree at the census tract level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_profess_tract <- function(year, survey) {  
  # Initialize an empty tibble
  profess_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    profession <- get_acs(geography = "tract",
                          variables = c(professional = "B15003_024"),
                          survey = survey,
                          output = "tidy",
                          state = uni_states[i],
                          year = year,
                          moe_level = 95
    )
    # Append the professional degree data to the existing tibble
    profess_data <- bind_rows(profess_data, profession)
  }
  # Return the final tibble with all professional degree data
  return(profess_data)
}


# Create a function to get the number who received Doctorate degree at the census tract level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_doctorate_tract <- function(year, survey) {  
  # Initialize an empty tibble
  doc_data <- tibble()
  
  # Loop through these unique states 
  for (i in 1:uni_state_num) {
    doctorate <- get_acs(geography = "tract",
                         variables = c(doctorate = "B15003_025"),
                         survey = survey,
                         output = "tidy",
                         state = uni_states[i],
                         year = year,
                         moe_level = 95
    )
    # Append the Doctorate degree data to the existing tibble
    doc_data <- bind_rows(doc_data, doctorate)
  }
  # Return the final tibble with all Doctorate's degree data
  return(doc_data)
}


# Initialize the empty tibbles to store the total number (denominator) and the respective number of people with the following degrees:
# associate's degree, bachelor's degree, master's degree, professional degree, doctorate degree
total_tract <- tibble()
associate_tract <- tibble()
bachelor_tract <- tibble()
master_tract <- tibble()
profession_tract <- tibble()
doctorate_tract <- tibble()
# Loop through these years and append the number of total educated and each education category to their respective tibble
for (j in year_list) {
  # Create two temporary tibbles to store the census data for this specific iteration (year)
  total_cur <- pull_total_tract(j, "acs5")
  associate_cur <- pull_associate_tract(j, "acs5")
  bachelor_cur <- pull_bachelor_tract(j, "acs5")   
  master_cur <- pull_master_tract(j, "acs5")
  profess_cur <- pull_profess_tract(j, "acs5")   
  doctorate_cur <- pull_doctorate_tract(j, "acs5")   #For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in the total_cur tibble and the educate_cur tibble
  total_cur <- total_cur %>% mutate(year = j)
  associate_cur <- associate_cur %>% mutate(year = j)
  bachelor_cur <- bachelor_cur %>% mutate(year = j)
  master_cur <- master_cur %>% mutate(year = j)
  profess_cur <- profess_cur %>% mutate(year = j)
  doctorate_cur <- doctorate_cur %>% mutate(year = j)
  
  # Append the temporary data and educate data to the existing tibble
  total_tract <- bind_rows(total_tract, total_cur)
  associate_tract <- bind_rows(associate_tract, associate_cur)
  bachelor_tract <- bind_rows(bachelor_tract, bachelor_cur)
  master_tract <- bind_rows(master_tract, master_cur)
  profession_tract <- bind_rows(profession_tract, profess_cur)
  doctorate_tract <- bind_rows(doctorate_tract, doctorate_cur)
}


# A check for whether the GEOID has the same values for all these tibbles
identical(total_tract$GEOID, associate_tract$GEOID)
identical(total_tract$GEOID, bachelor_tract$GEOID)
identical(total_tract$GEOID, master_tract$GEOID)
identical(total_tract$GEOID, profession_tract$GEOID)
identical(total_tract$GEOID, doctorate_tract$GEOID)

# Create another tibble for proportion of education (sum of numbers of each education degrees / total)
educate_tract <- tibble (FIPS_Census_Tract = total_tract$GEOID,
                   NAME = total_tract$NAME,
                   variable = "education_prop",
                   estimate = ((associate_tract$estimate + bachelor_tract$estimate + master_tract$estimate + profession_tract$estimate + doctorate_tract$estimate) / total_tract$estimate),
                   year = total_tract$year
)


# Join the educate and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
education_tract <- inner_join(educate_tract, fips_nums_cleaned, by = c("FIPS_Census_Tract", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# education_tract <- inner_join(educate_tract, fips_nums_cleaned, by = "FIPS_Census_Tract") %>%
#                                                 select(cohortPatientId, variable, estimate, year)


# Sanity check function (the estimates for education should not be greater than 1, because it is a proportion)
if (!all(education_tract$estimate <= 1, na.rm=TRUE)) {
  print("Education proportion should not be greater than 1")
} else {
  print("All education proportions are smaller or equal to 1")
}



########################################
#### Proportion of education - ZCTA ####
########################################

# We need separate functions for total (denominator), 
# the number of people who received associate's degree, 
# the number of people who received Bachelor's degree,
# the number of people who received Master's degree,
# the number of people who received Professional degree,
# the number of people who received Doctorate degree


# Create a function to get the required census estimates at the ZCTA level for the total (denominator)
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_total_zcta <- function(year, survey) {  
  # Initialize an empty tibble
  total_data <- tibble()
  
  total_data <- get_acs(geography = "zcta",
                    variables = c(total = "B15003_001"),
                    survey = survey,
                    output = "tidy",
                    year = year,
                    moe_level = 95
  )

  # Return the final tibble with all total data
  return(total_data)
}


# Create a function to get the number who received associate's degree at the ZCTA level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_associate_zcta <- function(year, survey) {  
  # Initialize an empty tibble
  associate_data <- tibble()

  associate_data <- get_acs(geography = "zcta",
                       variables = c(associate = "B15003_021"),
                       survey = survey,
                       output = "tidy",
                       year = year,
                       moe_level = 95
  )
  
  # Return the final tibble with all associate's degree data
  return(associate_data)
}


# Create a function to get the number who received Bachelor's degree at the ZCTA level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_bachelor_zcta <- function(year, survey) {  
  # Initialize an empty tibble
  bachelor_data <- tibble()
  
  bachelor_data <- get_acs(geography = "zcta",
                      variables = c(bachelor = "B15003_022"),
                      survey = survey,
                      output = "tidy",
                      year = year,
                      moe_level = 95
  )

  # Return the final tibble with all Bachelor's degree data
  return(bachelor_data)
}


# Create a function to get the number who received Master's degree at the ZCTA level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_master_zcta <- function(year, survey) {  
  # Initialize an empty tibble
  master_data <- tibble()
  
  master_data <- get_acs(geography = "zcta",
                      variables = c(master = "B15003_023"),
                      survey = survey,
                      output = "tidy",
                      year = year,
                      moe_level = 95
  )

  # Return the final tibble with all Master's degree data
  return(master_data)
}


# Create a function to get the number who received professional degree at the ZCTA level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_profess_zcta <- function(year, survey) {  
  # Initialize an empty tibble
  profess_data <- tibble()
  
  profess_data <- get_acs(geography = "zcta",
                        variables = c(professional = "B15003_024"),
                        survey = survey,
                        output = "tidy",
                        year = year,
                        moe_level = 95
  )
  
  # Return the final tibble with all professional degree data
  return(profess_data)
}


# Create a function to get the number who received Doctorate degree at the ZCTA level
# "year" should be the year of census data, and it should be a number (e.g., 2021); 
# "survey" should be "acs1", "acs3", or "acs5"
pull_doctorate_zcta <- function(year, survey) {  
  # Initialize an empty tibble
  doc_data <- tibble()
  
  doc_data <- get_acs(geography = "zcta",
                         variables = c(doctorate = "B15003_025"),
                         survey = survey,
                         output = "tidy",
                         year = year,
                         moe_level = 95
  )

  # Return the final tibble with all Doctorate's degree data
  return(doc_data)
}


# Initialize the empty tibbles to store the total number (denominator) and the respective number of people with the following degrees:
# associate's degree, bachelor's degree, master's degree, professional degree, doctorate degree
total_zcta <- tibble()
associate_zcta <- tibble()
bachelor_zcta <- tibble()
master_zcta <- tibble()
profession_zcta <- tibble()
doctorate_zcta <- tibble()
# Loop through these years and append the number of total educated and each education category to their respective tibble
for (j in year_list) {
  # Create two temporary tibbles to store the census data for this specific iteration (year)
  total_cur <- pull_total_zcta(j, "acs5")
  associate_cur <- pull_associate_zcta(j, "acs5")
  bachelor_cur <- pull_bachelor_zcta(j, "acs5")   
  master_cur <- pull_master_zcta(j, "acs5")
  profess_cur <- pull_profess_zcta(j, "acs5")   
  doctorate_cur <- pull_doctorate_zcta(j, "acs5")   #For NA-ACCORD, we use "acs5". You could also change the argument to "acs1" or "acs3", but some years may not have estimates in 1-year or 3-year.
  
  # Add year as a column in the total_cur tibble and the educate_cur tibble
  total_cur <- total_cur %>% mutate(year = j)
  associate_cur <- associate_cur %>% mutate(year = j)
  bachelor_cur <- bachelor_cur %>% mutate(year = j)
  master_cur <- master_cur %>% mutate(year = j)
  profess_cur <- profess_cur %>% mutate(year = j)
  doctorate_cur <- doctorate_cur %>% mutate(year = j)
  
  # Append the temporary data and educate data to the existing tibble
  total_zcta <- bind_rows(total_zcta, total_cur)
  associate_zcta <- bind_rows(associate_zcta, associate_cur)
  bachelor_zcta <- bind_rows(bachelor_zcta, bachelor_cur)
  master_zcta <- bind_rows(master_zcta, master_cur)
  profession_zcta <- bind_rows(profession_zcta, profess_cur)
  doctorate_zcta <- bind_rows(doctorate_zcta, doctorate_cur)
}


# A check for whether the GEOID has the same values for all these tibbles
identical(total_zcta$GEOID, associate_zcta$GEOID)
identical(total_zcta$GEOID, bachelor_zcta$GEOID)
identical(total_zcta$GEOID, master_zcta$GEOID)
identical(total_zcta$GEOID, profession_zcta$GEOID)
identical(total_zcta$GEOID, doctorate_zcta$GEOID)

# Create another tibble for proportion of education (sum of numbers of each education degrees / total)
educate_zcta <- tibble (ZIP = total_zcta$GEOID,
                         NAME = total_zcta$NAME,
                         variable = "education_prop",
                         estimate = ((associate_zcta$estimate + bachelor_zcta$estimate + master_zcta$estimate + profession_zcta$estimate + doctorate_zcta$estimate) / total_zcta$estimate),
                         year = total_zcta$year
)

# For year 2012, because the ZIP codes are different where they have 7 digits and only the last 5 digits are actually the ZIPs, we will extract them
# This does not seem to occur from year 2013-2021, but if future years have the same situation as year 2012, same thing needs to be done
zips_edu_2012 <- substr(educate_zcta$ZIP,3,7)
educate_zcta <- educate_zcta %>% 
  mutate(ZIP = ifelse(year==2012, zips_edu_2012, ZIP))

# Extract the ZCTAs
# zcta <- substr(educate_zcta$NAME,7,11)

# Join the educate and fips_num_cleaned tibbles by matching the "FIPS_Census_Tract" variable
# If the pulled census data have census tract FIPS codes matched to that of geocoded input addresses, the estimates will be retained
education_zcta <- inner_join(educate_zcta, fips_nums_cleaned, by = c("ZIP", "year"))

# The version with protected health information removed - only keep the pseudo id, variable, estimate, moe, and year
# education_zcta <- inner_join(educate_zcta, fips_nums_cleaned, by = "ZIP") %>%
#                                                 select(cohortPatientId, variable, estimate, year)


# Sanity check function (the estimates for education should not be greater than 1, because it is a proportion)
if (!all(education_zcta$estimate <= 1, na.rm=TRUE)) {
  print("Education proportion should not be greater than 1")
} else {
  print("All education proportions are smaller or equal to 1")
}




##### Generate the output of all three variables (median household income, employment proportion, education proportion) at four levels (state, county, census tract, zcta)
# Rename the estimate names for each tibble
# State level estimates
median_income_state <- median_income_state %>% rename(mStateIncome = estimate) # For median household income
employment_state <- employment_state %>% rename(pStateEmployed = estimate) # For employment proportion
education_state <- education_state %>% rename(pStateCollege = estimate) # For education proportion
# County level estimates
median_income_county <- median_income_county %>% rename(mCountyIncome = estimate) # For median household income
employment_county <- employment_county %>% rename(pCountyEmployed = estimate) # For employment proportion
education_county <- education_county %>% rename(pCountyCollege= estimate) # For education proportion
# Census tract level estimates
median_income_tract <- median_income_tract %>% rename(mTractIncome = estimate) # For median household income
employment_tract <- employment_tract %>% rename(pTractEmployed = estimate) # For employment proportion
education_tract <- education_tract %>% rename(pTractCollege = estimate) # For education proportion
# ZCTA level estimates
median_income_zcta <- median_income_zcta %>% rename(mZCTAIncome = estimate) # For median household income
employment_zcta <- employment_zcta %>% rename(pZCTAEmployed = estimate) # For employment proportion
education_zcta <- education_zcta %>% rename(pZCTACollege = estimate) # For education proportion


# # Converting variables from an atomic vector into a dataframe
# cohortPatientId <- tibble(cohortPatientId = median_income_state$cohortPatientId)


# Usually, state level estimates will have the greatest number of observations, because it is the most general geographical level
# Use state-level dataframes as references, check whether the other geographical levels (i.e., county, census tract, ZCTA) have non-matched observations






#############################################################################
################### Median income
missing_income_county <- anti_join(median_income_state, median_income_county, by = c("cohortPatientId", "year"))
missing_income_tract <- anti_join(median_income_state, median_income_tract, by = c("cohortPatientId", "year"))
missing_income_zcta <- anti_join(median_income_state, median_income_zcta, by = c("cohortPatientId", "year"))

## County
if (nrow(missing_income_county != 0)) {
  fill_income_county <- data.frame(FIPS_County = missing_income_county$FIPS_County,
                            NAME = missing_income_county$NAME,
                            variable = missing_income_county$variable,
                            mCountyIncome = NA,
                            moe = NA,
                            year = missing_income_county$year,
                            cohortPatientId = missing_income_county$cohortPatientId,
                            FIPS_State = missing_income_county$FIPS_State,
                            FIPS_Census_Tract = missing_income_county$FIPS_Census_Tract,
                            FIPS_Block = missing_income_county$FIPS_Block,
                            ZIP = missing_income_county$ZIP,
                            month = missing_income_county$month,
                            day = missing_income_county$day
                  )
  # Combine the newly created rows with the existing data frame
  median_income_county <- bind_rows(median_income_county, fill_income_county)
}

## Census tract
if (nrow(missing_income_tract != 0)) {
  fill_income_tract <- data.frame(FIPS_Census_Tract = missing_income_tract$FIPS_Census_Tract,
                            NAME = missing_income_tract$NAME,
                            variable = missing_income_tract$variable,
                            mTractIncome = NA,
                            moe = NA,
                            year = missing_income_tract$year,
                            cohortPatientId = missing_income_tract$cohortPatientId,
                            FIPS_State = missing_income_tract$FIPS_State,
                            FIPS_County = missing_income_tract$FIPS_County,
                            FIPS_Block = missing_income_tract$FIPS_Block,
                            ZIP = missing_income_tract$ZIP,
                            month = missing_income_tract$month,
                            day = missing_income_tract$day
  )
  # Combine the newly created rows with the existing data frame
  median_income_tract <- bind_rows(median_income_tract, fill_income_tract)
}

## ZCTA
if (nrow(missing_income_zcta != 0)) {
  fill_income_zcta <- data.frame(ZIP = missing_income_zcta$ZIP,
                           NAME = missing_income_zcta$NAME,
                           variable = missing_income_zcta$variable,
                           mZCTAIncome = NA,
                           moe = NA,
                           year = missing_income_zcta$year,
                           cohortPatientId = missing_income_zcta$cohortPatientId,
                           FIPS_State = missing_income_zcta$FIPS_State,
                           FIPS_County = missing_income_zcta$FIPS_County,
                           FIPS_Census_Tract = missing_income_zcta$FIPS_Census_Tract,
                           FIPS_Block = missing_income_zcta$FIPS_Block,
                           month = missing_income_zcta$month,
                           day = missing_income_zcta$day
  )
  # Combine the newly created rows with the existing data frame
  median_income_zcta <- bind_rows(median_income_zcta, fill_income_zcta)
}


################### Employment proportion
missing_employment_county <- anti_join(employment_state, employment_county, by = c("cohortPatientId", "year"))
missing_employment_tract <- anti_join(employment_state, employment_tract, by = c("cohortPatientId", "year"))
missing_employment_zcta <- anti_join(employment_state, employment_zcta, by = c("cohortPatientId", "year"))

## County
if (nrow(missing_employment_county != 0)) {
  fill_employment_county <- data.frame(FIPS_County = missing_employment_county$FIPS_County,
                            NAME = missing_employment_county$NAME,
                            variable = missing_employment_county$variable,
                            pCountyEmployed = NA,
                            year = missing_employment_county$year,
                            cohortPatientId = missing_employment_county$cohortPatientId,
                            FIPS_State = missing_employment_county$FIPS_State,
                            FIPS_Census_Tract = missing_employment_county$FIPS_Census_Tract,
                            FIPS_Block = missing_employment_county$FIPS_Block,
                            ZIP = missing_employment_county$ZIP,
                            month = missing_employment_county$month,
                            day = missing_employment_county$day
  )
  # Combine the newly created rows with the existing data frame
  employment_county<- bind_rows(employment_county, fill_employment_county)
}

## Census tract
if (nrow(missing_employment_tract != 0)) {
  fill_employment_tract <- data.frame(FIPS_Census_Tract = missing_employment_tract$FIPS_Census_Tract,
                                  NAME = missing_employment_tract$NAME,
                                  variable = missing_employment_tract$variable,
                                  pTractEmployed = NA,
                                  year = missing_employment_tract$year,
                                  cohortPatientId = missing_employment_tract$cohortPatientId,
                                  FIPS_State = missing_employment_tract$FIPS_State,
                                  FIPS_County = missing_employment_tract$FIPS_County,
                                  FIPS_Block = missing_employment_tract$FIPS_Block,
                                  ZIP = missing_employment_tract$ZIP,
                                  month = missing_employment_tract$month,
                                  day = missing_employment_tract$day
  )
  # Combine the newly created rows with the existing data frame
  employment_tract <- bind_rows(employment_tract, fill_employment_tract)
}

## ZCTA
if (nrow(missing_employment_zcta != 0)) {
  fill_employment_zcta <- data.frame(ZIP = missing_employment_zcta$ZIP,
                                      NAME = missing_employment_zcta$NAME,
                                      variable = missing_employment_zcta$variable,
                                      pZCTAEmployed = NA,
                                      year = missing_employment_zcta$year,
                                      cohortPatientId = missing_employment_zcta$cohortPatientId,
                                      FIPS_State = missing_employment_zcta$FIPS_State,
                                      FIPS_County = missing_employment_zcta$FIPS_County,
                                      FIPS_Census_Tract = missing_employment_zcta$FIPS_Census_Tract,
                                      FIPS_Block = missing_employment_zcta$FIPS_Block,
                                      month = missing_employment_zcta$month,
                                      day = missing_employment_zcta$day
  )
  # Combine the newly created rows with the existing data frame
  employment_zcta <- bind_rows(employment_zcta, fill_employment_zcta)
}


################### Education proportion
missing_education_county <- anti_join(education_state, education_county, by = c("cohortPatientId", "year"))
missing_education_tract <- anti_join(education_state, education_tract, by = c("cohortPatientId", "year"))
missing_education_zcta <- anti_join(education_state, education_zcta, by = c("cohortPatientId", "year"))

## County
if (nrow(missing_education_county != 0)) {
  fill_education_county <- data.frame(FIPS_County = missing_education_county$FIPS_County,
                                       NAME = missing_education_county$NAME,
                                       variable = missing_education_county$variable,
                                       pCountyCollege= NA,
                                       year = missing_education_county$year,
                                       cohortPatientId = missing_education_county$cohortPatientId,
                                       FIPS_State = missing_education_county$FIPS_State,
                                       FIPS_Census_Tract = missing_education_county$FIPS_Census_Tract,
                                       FIPS_Block = missing_education_county$FIPS_Block,
                                       ZIP = missing_education_county$ZIP,
                                       month = missing_education_county$month,
                                       day = missing_education_county$day
  )
  # Combine the newly created rows with the existing data frame
  education_county<- bind_rows(education_county, fill_education_county)
}

## Census tract
if (nrow(missing_education_tract != 0)) {
  fill_education_tract <- data.frame(FIPS_Census_Tract = missing_education_tract$FIPS_Census_Tract,
                                      NAME = missing_education_tract$NAME,
                                      variable = missing_education_tract$variable,
                                      pTractCollege = NA,
                                      year = missing_education_tract$year,
                                      cohortPatientId = missing_education_tract$cohortPatientId,
                                      FIPS_State = missing_education_tract$FIPS_State,
                                      FIPS_County = missing_education_tract$FIPS_County,
                                      FIPS_Block = missing_education_tract$FIPS_Block,
                                      ZIP = missing_education_tract$ZIP,
                                      month = missing_education_tract$month,
                                      day = missing_education_tract$day
  )
  # Combine the newly created rows with the existing data frame
  education_tract <- bind_rows(education_tract, fill_education_tract)
}

## ZCTA
if (nrow(missing_education_zcta != 0)) {
  fill_education_zcta <- data.frame(ZIP = missing_education_zcta$ZIP,
                                     NAME = missing_education_zcta$NAME,
                                     variable = missing_education_zcta$variable,
                                     pZCTACollege = NA,
                                     year = missing_education_zcta$year,
                                     cohortPatientId = missing_education_zcta$cohortPatientId,
                                     FIPS_State = missing_education_zcta$FIPS_State,
                                     FIPS_County = missing_education_zcta$FIPS_County,
                                     FIPS_Census_Tract = missing_education_zcta$FIPS_Census_Tract,
                                     FIPS_Block = missing_education_zcta$FIPS_Block,
                                     month = missing_education_zcta$month,
                                     day = missing_education_zcta$day
  )
  # Combine the newly created rows with the existing data frame
  education_zcta <- bind_rows(education_zcta, fill_education_zcta)
}


# Rename the NAME variable for each tibble
# State level estimates
median_income_state <- median_income_state %>% rename(State_NAME = NAME) # For median household income
employment_state <- employment_state %>% rename(State_NAME = NAME) # For employment proportion
education_state <- education_state %>% rename(State_NAME = NAME) # For education proportion
# County level estimates
median_income_county <- median_income_county %>% rename(County_NAME = NAME) # For median household income
employment_county <- employment_county %>% rename(County_NAME = NAME) # For employment proportion
education_county <- education_county %>% rename(County_NAME = NAME) # For education proportion
# Census tract level estimates
median_income_tract <- median_income_tract %>% rename(Tract_NAME = NAME) # For median household income
employment_tract <- employment_tract %>% rename(Tract_NAME = NAME) # For employment proportion
education_tract <- education_tract %>% rename(Tract_NAME = NAME) # For education proportion
# ZCTA level estimates
median_income_zcta <- median_income_zcta %>% rename(ZCTA_NAME = NAME) # For median household income
employment_zcta <- employment_zcta %>% rename(ZCTA_NAME = NAME) # For employment proportion
education_zcta <- education_zcta %>% rename(ZCTA_NAME = NAME) # For education proportion



# Remove the redundant variables
# income_county_modified <- median_income_county %>% select(-variable, -moe, -FIPS_State, -FIPS_County, -FIPS_Block, -ZIP, -month, -day, -cohortPatientId)
# income_tract_modified <- median_income_tract %>% select(-variable, -moe, -FIPS_State, -FIPS_County, -FIPS_Block, -ZIP, -month, -day, -cohortPatientId)
# income_zcta_modified <- median_income_zcta %>% select(-variable, -moe, -FIPS_State, -FIPS_County, -FIPS_Block, -ZIP, -month, -day, -cohortPatientId)
# 
# employ_state_modified <- employment_state %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Block, -State_NAME, -ZIP, -month, -day, -cohortPatientId)
# employ_county_modified <- employment_county %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Block, -County_NAME, -ZIP, -month, -day, -cohortPatientId)
# employ_tract_modified <- employment_tract %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Block, -Tract_NAME, -ZIP, -month, -day, -cohortPatientId)
# employ_zcta_modified <- employment_zcta %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Block, -ZCTA_NAME, -ZIP, -month, -day, -cohortPatientId)
# 
# educate_state_modified <- education_state %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Block, -State_NAME, -ZIP, -month, -day, -cohortPatientId)
# educate_county_modified <- education_county %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Block, -County_NAME, -ZIP, -month, -day, -cohortPatientId)
# educate_tract_modified <- education_tract %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Block, -Tract_NAME, -ZIP, -month, -day, -cohortPatientId)
# educate_zcta_modified <- education_zcta %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Block, -ZCTA_NAME, -ZIP, -month, -day, -cohortPatientId)

income_county_modified <- median_income_county %>% select(-variable, -moe, -FIPS_State, -FIPS_County, -FIPS_Census_Tract)
income_tract_modified <- median_income_tract %>% select(-variable, -moe, -FIPS_State, -FIPS_County, -FIPS_Census_Tract)
income_zcta_modified <- median_income_zcta %>% select(-variable, -moe, -FIPS_State, -FIPS_County, -FIPS_Census_Tract)

employ_state_modified <- employment_state %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Census_Tract, -State_NAME)
employ_county_modified <- employment_county %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Census_Tract, -County_NAME)
employ_tract_modified <- employment_tract %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Census_Tract, -Tract_NAME)
employ_zcta_modified <- employment_zcta %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Census_Tract, -ZCTA_NAME)

educate_state_modified <- education_state %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Census_Tract, -State_NAME)
educate_county_modified <- education_county %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Census_Tract, -County_NAME)
educate_tract_modified <- education_tract %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Census_Tract, -Tract_NAME)
educate_zcta_modified <- education_zcta %>% select(-variable, -FIPS_State, -FIPS_County, -FIPS_Census_Tract, -ZCTA_NAME)


# Merge all the geographical levels' all three estimates into one large tibble
output <- inner_join(median_income_state, income_county_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))
output <- inner_join(output, income_tract_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))
output <- inner_join(output, income_zcta_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))

output <- inner_join(output, employ_state_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))
output <- inner_join(output, employ_county_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))
output <- inner_join(output, employ_tract_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))
output <- inner_join(output, employ_zcta_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))

output <- inner_join(output, educate_state_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))
output <- inner_join(output, educate_county_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))
output <- inner_join(output, educate_tract_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))
output <- inner_join(output, educate_zcta_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day"))


# # Import the csv file for the randomly generated cluster numbers at state, county, census tract, and ZCTA level
# cluster_nums <- read.csv("cluster_nums.csv") 
# 
# # Because csv files automatically remove the leading zeros, we will have to refill them here
# state_clusters <- sprintf("%02s", cluster_nums$cluster_state)
# cluster_nums$cluster_state <- state_clusters
# county_clusters <- sprintf("%05s", cluster_nums$cluster_county)
# cluster_nums$cluster_county <- county_clusters
# tract_clusters <- sprintf("%05s", cluster_nums$cluster_tract)
# cluster_nums$cluster_tract <- tract_clusters
# zcta_clusters <- sprintf("%05s", cluster_nums$cluster_zcta)
# cluster_nums$cluster_zcta <- zcta_clusters
# fips_clusters <- sprintf("%011s", cluster_nums$FIPS_Census_Tract)
# cluster_nums$FIPS_Census_Tract <- fips_clusters
# 
# 
# # Remove the "cohortPatientId" in the cluster number tibble to avoid redundancy
# cleaned_cluster <- cluster_nums %>% select(-cohortPatientId)
# 
# # Merge random cluster numbers for the state, county, census tract, and zcta into the output tibble
# output <- inner_join(output, cleaned_cluster, by = c("FIPS_Census_Tract", "year"))

# Reduce the number of decimal places of the employment proportion estimates and the education proportion estimates
# There are actually hidden decimal places 
# Merge all the geographical levels' all three estimates into one large tibble
output <- inner_join(median_income_state, income_county_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))
output <- inner_join(output, income_tract_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))
output <- inner_join(output, income_zcta_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))

output <- inner_join(output, employ_state_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))
output <- inner_join(output, employ_county_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))
output <- inner_join(output, employ_tract_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))
output <- inner_join(output, employ_zcta_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))

output <- inner_join(output, educate_state_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))
output <- inner_join(output, educate_county_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))
output <- inner_join(output, educate_tract_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))
output <- inner_join(output, educate_zcta_modified, by = c("FIPS_Block", "ZIP", "cohortPatientId", "year", "month", "day","start","end","record_date"))

# Export the output for running the "perturbed_digit.R"
write.csv(output, file = "temp_output.csv", row.names = FALSE)
